/* 
 * Copyright (C) 2024 Petr Mironychev
 *
 * This file is part of QodeAssist.
 *
 * QodeAssist is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * QodeAssist is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with QodeAssist. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#include <QJsonArray>

#include "llmcore/PromptTemplate.hpp"

namespace QodeAssist::Templates {

class OllamaAutoFim : public LLMCore::PromptTemplate
{
public:
    LLMCore::TemplateType type() const override { return LLMCore::TemplateType::Fim; }
    QString name() const override { return "Ollama Auto FIM"; }
    QString promptTemplate() const override { return {}; }
    QStringList stopWords() const override { return QStringList(); }
    void prepareRequest(QJsonObject &request, const LLMCore::ContextData &context) const override
    {
        request["prompt"] = context.prefix;
        request["suffix"] = context.suffix;
    }
    QString description() const override { return "template will take from ollama modelfile"; }
};

class OllamaAutoChat : public LLMCore::PromptTemplate
{
public:
    LLMCore::TemplateType type() const override { return LLMCore::TemplateType::Chat; }
    QString name() const override { return "Ollama Auto Chat"; }
    QString promptTemplate() const override { return {}; }
    QStringList stopWords() const override { return QStringList(); }

    void prepareRequest(QJsonObject &request, const LLMCore::ContextData &context) const override
    {
    }
    QString description() const override { return "template will take from ollama modelfile"; }
};

} // namespace QodeAssist::Templates
